import wireframe
import pygame

class ProjectionViewer:
    """diplays 3d objects"""

    
    key_to_function = {
        #Session5
        pygame.K_q: (lambda x: x.rotateAll('X', 0.1)),
        pygame.K_w: (lambda x: x.rotateAll('X', -0.1)),
        pygame.K_a: (lambda x: x.rotateAll('Y', 0.1)),
        pygame.K_s: (lambda x: x.rotateAll('Y', -0.1)),
        pygame.K_z: (lambda x: x.rotateAll('Z', 0.1)),
        pygame.K_x: (lambda x: x.rotateAll('Z', -0.1)),

        pygame.K_LEFT: (lambda x: x.translateAll('x',-10)),
        pygame.K_RIGHT: (lambda x: x.translateAll('x', 10)),
        pygame.K_DOWN: (lambda x: x.translateAll('Y', 10)),
        pygame.K_UP: (lambda x: x.translateAll('Y', -10)),
        pygame.K_EQUALS: (lambda x: x.scaleAll(1.25)),
        pygame.K_MINUS: (lambda x: x.scaleAll(0.8)),
    }

    def __init__(self,width,height):
        self.width= width
        self.height = height
        self.screen = pygame.display.set_mode((width, height))
        pygame.display.set_caption("Wireframe Display")
        self.background = (10,10,50)

        self.wireframes={}
        self.displayNodes = True
        self.displayEdges = True
        self.nodeColour = (255,255,255)
        self.edgeColour = (200,200,200)
        self.nodeRadius = 4


    def run(self):
        running= True
        while running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running=False
                #add these after
                elif event.type == pygame.KEYDOWN:
                    if event.key in ProjectionViewer.key_to_function:
                        ProjectionViewer.key_to_function[event.key](self)

            self.screen.fill(self.background)
            #add this in step 5
            self.display()
            pygame.display.flip()

    def addWireframe(self, name, wireframe):
        self.wireframes[name] = wireframe

    
    def display(self):
        self.screen.fill(self.background)
        for wireframe in self.wireframes.values():
            if self.displayEdges:
                for edge in wireframe.edges:
                    pygame.draw.aaline(self.screen, self.edgeColour, (edge.start.x, edge.start.y), (edge.stop.x, edge.stop.y), 1)

            if self.displayNodes:
                for node in wireframe.nodes:
                    pygame.draw.circle(self.screen,self.nodeColour, (int(node.x), int(node.y)), self.nodeRadius, 0)
                    #do not forget to add to while running the call to self.display()

    
    def translateAll(self, axis, d):
        for wireframe in self.wireframes.values():
            wireframe.translate(axis,d)

    def scaleAll(self, scale):
        center_x = self.width/2
        center_y = self.height/2
        center_xcenter_y=center_x,center_y
        for wireframe in self.wireframes.values():
            wireframe.scale(center_xcenter_y, scale)

    #Session5
    def rotateAll(self,axis,theta):
        rotateFunction = 'rotate' + axis
        for wireframe in self.wireframes.values():
            center = wireframe.findCenter()
            getattr(wireframe, rotateFunction)(center, theta)



if __name__ == '__main__':
    pv = ProjectionViewer(400,300)

    #add these in a later step - COPY PASTE from other file - STEP2 
    cube = wireframe.Wireframe()
    cube_nodes = [(x,y,z) for x in (0,1) for y in (0,1) for z in (0,1)]
    #step5 replace the previous line with
    cube_nodes = [(x,y,z) for x in (50,250) for y in (50,250) for z in (50,250)]
    cube.addNodes(cube_nodes)
    cube.addEdges([(n,n+4) for n in range(0,4)])
    cube.addEdges([(n,n+1) for n in range(0,8,2)])
    cube.addEdges([(n,n+2) for n in (0,1,4,5)])
    pv.addWireframe('cube', cube)
    # #################################


    pv.run()